#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Tools for converting LaTeX to unicode.

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import re
import string


from . import NAME_SMALL, SMALL


def latex_to_unicode(text, capitalize=False):
    """Replace LaTeX entities with the equivalent unicode and optionally capitalize.

    :param text: The LaTeX string to be converted
    :param capitalize: Can be 'sentence', 'name', 'title', 'upper', 'lower'
    """
    if capitalize:
        res = []
        brac_count = 0
        for i, c in enumerate(text):
            if c == "{":
                brac_count += 1
            if c == "}":
                brac_count -= 1
            if brac_count > 0:
                res.append(c)
            elif capitalize == "upper" or (i == 0 and not capitalize == "lower"):
                res.append(c.upper())
            elif capitalize == "sentence" and (
                i > 2 and text[i - 1] == " " and text[i - 2] == "."
            ):
                res.append(c.upper())
            elif (capitalize == "name" and text[i - 1] in [" ", "-"]) or (
                capitalize == "title" and text[i - 1] == " "
            ):
                nextword = text[i:].split(" ", 1)[0].rstrip(string.punctuation)
                nextword = nextword[:1].lower() + nextword[1:] if text else ""
                if capitalize == "name" and nextword in NAME_SMALL:
                    res.append(c.lower())
                elif capitalize == "title" and nextword in SMALL:
                    res.append(c.lower())
                else:
                    res.append(c.upper())
            elif capitalize == "name" and c == c.upper():
                n1 = text[i - 1] if i > 0 else None
                n2 = text[i - 2] if i > 1 else None
                n3 = text[i - 3] if i > 2 else None
                n4 = text[i - 4] if i > 3 else None
                if n2 == "M" and n1 == "c" and (not n3 or n3 == " "):
                    res.append(c)  # McCartney
                elif n2 == "O" and n1 == "'" and (not n3 or n3 == " "):
                    res.append(c)  # O'Boyle
                elif n3 == "M" and n2 == "a" and n1 == "c" and (not n4 or n4 == " "):
                    res.append(c)  # MacGarry
                else:
                    res.append(c.lower())
            else:
                res.append(c.lower())
        text = "".join(res)
    if any(i in text for i in ["\\", "{", "}", "$", "&", "%", "#", "_"]):
        for k, v in LATEX_MAPPINGS.items():
            text = text.replace(k, v)
        for k, v in LATEX_SUB_MAPPINGS.items():
            text = text.replace(k, v)
        for mod in [
            "mathbb",
            "mathbf",
            "mathbit",
            "mathfrak",
            "mathrm",
            "mathscr",
            "mathsf",
            "mathsfbf",
            "mathsfbfsl",
            "mathsfsl",
            "mathsl",
            "mathslbb",
            "mathtt",
        ]:
            text = re.sub(r"\\%s\{([\\\w]+)\}" % mod, r"\1", text)
        for k, v in LATEX_SUB_SUB_MAPPINGS.items():
            text = text.replace(k, v)
        for k, v in LATEX_COMBINING_CHARS.items():
            text = re.sub(r"%s\{?(\w)\}?" % k, r"\1%s" % v, text)
        text = re.sub(r"\\noopsort\{.*?\}", r"", text)
        text = re.sub(r"\\path\|(.*?)\|", r"\1", text)
        text = re.sub(r"(?<!\\)[{}$]", r"", text)
        text = re.sub(r"\\([{}$&_])", r"\1", text)
    return text


# The LaTeX characters apply to the subsequent character, while the unicode applies to the previous character
# The LaTeX keys are regular expressions
LATEX_COMBINING_CHARS = {
    r"\\=": "\u0304",
    r"\\'": "\u0301",
    r"\\\"": "\u0308",
    r"\\\.": "\u0307",
    r"\\\^": "\u0302",
    r"\\~": "\u0303",
    r"\\`": "\u0300",
    r"\\cyrchar\\C": "\u030f",
    r"\\c": "\u0327",
    r"\\H": "\u030b",
    r"\\r": "\u030a",
    r"\\u": "\u0306",
    r"\\v": "\u030c",
    r"\\k": "\u0328",
}

# These mapping are substring of other mappings, so must ensure they are done last
LATEX_SUB_MAPPINGS = {
    "{{/}\\!\\!{/}}": "\u2afd",
    "\\-": "\u00ad",  # Soft hyphen (hyphenation "hint") - removed by normalize function
    "\\approx ": "\u2248",
    "\\cal": "",
    "\\cong ": "\u2245",
    "\\ddot{\\iota}": "\u03ca",
    "\\ddot{\\upsilon}": "\u03cb",
    "\\em ": "",
    "\\emph": "",
    "\\equiv ": "\u2261",
    "\\geq ": "\u2265",
    "\\hspace{-.9ex}": "",
    "\\hspace{.2ex}": "",
    "\\in ": "\u2208",
    "\\int\\!\\int ": "\u222c",
    "\\it ": "",
    "\\leq ": "\u2264",
    "\\mbox": "",
    "\\mu": "\u03bc",
    "\\ni ": "\u220b",
    "\\prec ": "\u227a",
    "\\preceq ": "\u2aaf",
    "\\raisebox{.5ex}": "",
    "\\raisebox{1.3ex}": "",
    "\\raisebox{2ex}": "",
    "\\raisebox{-.3ex}": "",
    "\\rotatebox[]{180}": "",
    "\\scriptsize ": "",
    "\\sim ": "\u223c",
    "\\simeq ": "\u2243",
    "\\sqsubseteq ": "\u2291",
    "\\sqsupseteq ": "\u2292",
    "\\subset ": "\u2282",
    "\\subseteq ": "\u2286",
    "\\succ ": "\u227b",
    "\\succeq ": "\u2ab0",
    "\\supset ": "\u2283",
    "\\supseteq ": "\u2287",
    "\\times ": "\u00d7 ",
    "\\tt ": "",
}

# Further substrings and greek letters that may appear after stripping math fonts
LATEX_SUB_SUB_MAPPINGS = {
    "\\i": "\u0131",
    "\\l": "\u0142",
    "\\L": "\u0141",
    "\\O": "\u00d8",
    "\\o": "\u00f8",
    "\\int ": "\u222b",
    "\\Alpha ": "\u0391",
    "\\alpha ": "\u03b1",
    "\\Beta ": "\u0392",
    "\\beta ": "\u03b2",
    "\\Chi ": "\u03a7",
    "\\chi ": "\u03c7",
    "\\Delta ": "\u0394",
    "\\delta ": "\u03b4",
    "\\Epsilon ": "\u0395",
    "\\epsilon ": "\u03b5",
    "\\Eta ": "\u0397",
    "\\eta ": "\u03b7",
    "\\Gamma ": "\u0393",
    "\\gamma ": "\u03b3",
    "\\Iota ": "\u0399",
    "\\iota ": "\u03b9",
    "\\Kappa ": "\u039a",
    "\\kappa ": "\u03ba",
    "\\Lambda ": "\u039b",
    "\\lambda ": "\u03bb",
    "\\nabla ": "\u2207",
    "\\Omega ": "\u03a9",
    "\\omega ": "\u03c9",
    "\\Phi ": "\u03a6",
    "\\phi ": "\u03d5",
    "\\Pi ": "\u03a0",
    "\\pi ": "\u03c0",
    "\\Psi ": "\u03a8",
    "\\psi ": "\u03c8",
    "\\Rho ": "\u03a1",
    "\\rho ": "\u03c1",
    "\\Sigma ": "\u03a3",
    "\\sigma ": "\u03c3",
    "\\Tau ": "\u03a4",
    "\\tau ": "\u03c4",
    "\\theta ": "\u03b8",
    "\\Theta ": "\u0398",
    "\\Upsilon ": "\u03a5",
    "\\upsilon ": "\u03c5",
    "\\varepsilon ": "\u025b",
    "\\varkappa ": "\u03f0",
    "\\varphi ": "\u03c6",
    "\\varpi ": "\u03d6",
    "\\varrho ": "\u03f1",
    "\\varsigma ": "\u03c2",
    "\\vartheta ": "\u03d1",
    "\\Xi ": "\u039e",
    "\\xi ": "\u03be",
    "\\Zeta ": "\u0396",
    "\\zeta ": "\u03b6",
}


# Unicode to LaTeX character mappings, derived from https://gist.github.com/piquadrat/798549
LATEX_MAPPINGS = {
    ",,": "\u201e",
    ":=": "\u2254",
    "''''": "\u2057",
    "{_\\ast}": "\u2217",
    "{'''}": "\u2034",
    "{''}": "\u2033",
    "{'}": "\u2032",
    "{\\LaTeX}": "LaTeX",
    "{\\TeX}": "TeX",
    "{\\fontencoding{LECO}\\selectfont\\char177}": "\u0311",
    "{\\fontencoding{LECO}\\selectfont\\char184}": "\u0318",
    "{\\fontencoding{LECO}\\selectfont\\char185}": "\u0319",
    "{\\fontencoding{LECO}\\selectfont\\char203}": "\u032b",
    "{\\fontencoding{LECO}\\selectfont\\char207}": "\u032f",
    "{\\fontencoding{LECO}\\selectfont\\char215}": "\u0337",
    "{\\fontencoding{LECO}\\selectfont\\char216}": "\u0338",
    "{\\fontencoding{LECO}\\selectfont\\char218}": "\u033a",
    "{\\fontencoding{LECO}\\selectfont\\char219}": "\u033b",
    "{\\fontencoding{LECO}\\selectfont\\char220}": "\u033c",
    "{\\fontencoding{LECO}\\selectfont\\char221}": "\u033d",
    "{\\fontencoding{LECO}\\selectfont\\char225}": "\u0361",
    "{\\fontencoding{LEIP}\\selectfont\\char202}": "\u027f",
    "{\\fontencoding{LEIP}\\selectfont\\char61}": "\u0258",
    "{\\fontencoding{LELA}\\selectfont\\char195}": "\u01ba",
    "{\\fontencoding{LELA}\\selectfont\\char201}": "\u013f",
    "{\\fontencoding{LELA}\\selectfont\\char202}": "\u0140",
    "{\\fontencoding{LELA}\\selectfont\\char40}": "\u0126",
    "{\\fontencoding{LELA}\\selectfont\\char47}": "\u0166",
    "{\\fontencoding{LELA}\\selectfont\\char63}": "\u0167",
    "{\\fontencoding{LELA}\\selectfont\\char91}": "\u0138",
    "{\\user}": "\u007e",
    "{\\rlap{\\textbackslash}{{/}\\!\\!{/}}}": "\u2afd\u20e5",
    "{^1}": "\u00b9",
    "{^2}": "\u00b2",
    "{^3}": "\u00b3",
    "\\\\": "\n",
    "\\_": "\u005f",
    "\\;": " ",  # \u2009?
    "\\.{C}": "\u010a",
    "\\.{c}": "\u010b",
    "\\.{E}": "\u0116",
    "\\.{e}": "\u0117",
    "\\.{G}": "\u0120",
    "\\.{g}": "\u0121",
    "\\.{I}": "\u0130",
    "\\.{Z}": "\u017b",
    "\\.{z}": "\u017c",
    "\\'{}{I}": "\u038a",
    "\\'{}O": "\u038c",
    "\\'{\\i}": "\u00ed",
    "\\'{$\\alpha$}": "\u03ac",
    "\\'{A}": "\u00c1",
    "\\'{a}": "\u00e1",
    "\\'{C}": "\u0106",
    "\\'{c}": "\u0107",
    "\\'{E}": "\u00c9",
    "\\'{e}": "\u00e9",
    "\\'{g}": "\u01f5",
    "\\'{H}": "\u0389",
    "\\'{I}": "\u00cd",
    "\\'{L}": "\u0139",
    "\\'{l}": "\u013a",
    "\\'{N}": "\u0143",
    "\\'{n}": "\u0144",
    "\\'{O}": "\u00d3",
    "\\'{o}": "\u00f3",
    "\\'{R}": "\u0154",
    "\\'{r}": "\u0155",
    "\\'{S}": "\u015a",
    "\\'{s}": "\u015b",
    "\\'{U}": "\u00da",
    "\\'{u}": "\u00fa",
    "\\'{Y}": "\u00dd",
    "\\'{y}": "\u00fd",
    "\\'{Z}": "\u0179",
    "\\'{z}": "\u017a",
    '\\"{\\i}': "\u00ef",
    '\\"{A}': "\u00c4",
    '\\"{a}': "\u00e4",
    '\\"{E}': "\u00cb",
    '\\"{e}': "\u00eb",
    '\\"{I}': "\u00cf",
    '\\"{O}': "\u00d6",
    '\\"{o}': "\u00f6",
    '\\"{U}': "\u00dc",
    '\\"{u}': "\u00fc",
    '\\"{y}': "\u00ff",
    '\\"{Y}': "\u0178",
    "\\&amp;": "\u0026",
    "\\#": "\u0023",
    "\\%": "\u0025",
    "\\`{\\i}": "\u00ec",
    "\\`{A}": "\u00c0",
    "\\`{a}": "\u00e0",
    "\\`{E}": "\u00c8",
    "\\`{e}": "\u00e8",
    "\\`{I}": "\u00cc",
    "\\`{O}": "\u00d2",
    "\\`{o}": "\u00f2",
    "\\`{U}": "\u00d9",
    "\\`{u}": "\u00f9",
    "\\^{}": "\u005e",
    "\\^{\\i}": "\u00ee",
    "\\^{\\j}": "\u0135",
    "\\^{A}": "\u00c2",
    "\\^{a}": "\u00e2",
    "\\^{C}": "\u0108",
    "\\^{c}": "\u0109",
    "\\^{E}": "\u00ca",
    "\\^{e}": "\u00ea",
    "\\^{G}": "\u011c",
    "\\^{g}": "\u011d",
    "\\^{H}": "\u0124",
    "\\^{h}": "\u0125",
    "\\^{I}": "\u00ce",
    "\\^{J}": "\u0134",
    "\\^{O}": "\u00d4",
    "\\^{o}": "\u00f4",
    "\\^{S}": "\u015c",
    "\\^{s}": "\u015d",
    "\\^{U}": "\u00db",
    "\\^{u}": "\u00fb",
    "\\^{W}": "\u0174",
    "\\^{w}": "\u0175",
    "\\^{Y}": "\u0176",
    "\\^{y}": "\u0177",
    "\\={\\i}": "\u012b",
    "\\={A}": "\u0100",
    "\\={a}": "\u0101",
    "\\={E}": "\u0112",
    "\\={e}": "\u0113",
    "\\={I}": "\u012a",
    "\\={O}": "\u014c",
    "\\={o}": "\u014d",
    "\\={U}": "\u016a",
    "\\={u}": "\u016b",
    "\\~{\\i}": "\u0129",
    "\\~{A}": "\u00c3",
    "\\~{a}": "\u00e3",
    "\\~{I}": "\u0128",
    "\\~{N}": "\u00d1",
    "\\~{n}": "\u00f1",
    "\\~{O}": "\u00d5",
    "\\~{o}": "\u00f5",
    "\\~{U}": "\u0168",
    "\\~{u}": "\u0169",
    "\\aa": "\u00e5",
    "\\AA": "\u212b",
    "\\acute{\\ddot{\\iota}}": "\u0390",
    "\\acute{\\ddot{\\upsilon}}": "\u03b0",
    "\\acute{\\epsilon}": "\u03ad",
    "\\acute{\\eta}": "\u03ae",
    "\\acute{\\iota}": "\u03af",
    "\\acute{\\omega}": "\u03ce",
    "\\acute{\\upsilon}": "\u03cd",
    "\\AE": "\u00c6",
    "\\ae": "\u00e6",
    "\\aleph ": "\u2135",
    "\\allequal ": "\u224c",
    "\\Alpha ": "\u0391",
    "\\alpha ": "\u03b1",
    "\\amalg ": "\u2a3f",
    "\\angle ": "\u2220",
    "\\Angle ": "\u299c",
    "\\approxeq ": "\u224a",
    "\\approxnotequal ": "\u2246",
    "\\aquarius ": "\u2652",
    "\\aries ": "\u2648",
    "\\arrowwaveright ": "\u219d",
    "\\ast ": "\u002a",
    "\\asymp ": "\u224d",
    "\\backepsilon ": "\u03f6",
    "\\backprime ": "\u2035",
    "\\backsim ": "\u223d",
    "\\backsimeq ": "\u22cd",
    "\\barwedge ": "\u2305",
    "\\because ": "\u2235",
    "\\Beta ": "\u0392",
    "\\beta ": "\u03b2",
    "\\beth ": "\u2136",
    "\\between ": "\u226c",
    "\\bigcap ": "\u22c2",
    "\\bigcirc ": "\u25ef",
    "\\bigcup ": "\u22c3",
    "\\bigodot ": "\u2299",
    "\\bigoplus ": "\u2295",
    "\\bigotimes ": "\u2297",
    "\\bigsqcup ": "\u2294",
    "\\bigstar ": "\u2605",
    "\\bigtriangledown ": "\u25bd",
    "\\bigtriangleup ": "\u25b3",
    "\\biguplus ": "\u228e",
    "\\bigvee ": "\u22c1",
    "\\bigwedge ": "\u22c0",
    "\\blacklozenge ": "\u29eb",
    "\\blacksquare ": "\u25aa",
    "\\blacktriangle ": "\u25b4",
    "\\blacktriangledown ": "\u25be",
    "\\blacktriangleleft ": "\u25c2",
    "\\blacktriangleright ": "\u25b8",
    "\\bot ": "\u22a5",
    "\\bowtie ": "\u22c8",
    "\\box ": "\u25a1",
    "\\boxdot ": "\u22a1",
    "\\boxminus ": "\u229f",
    "\\boxplus ": "\u229e",
    "\\boxtimes ": "\u22a0",
    "\\bullet ": "\u2219",
    "\\Bumpeq ": "\u224e",
    "\\bumpeq ": "\u224f",
    "\\c{}": "\u00b8",
    "\\c{C}": "\u00c7",
    "\\c{c}": "\u00e7",
    "\\c{G}": "\u0122",
    "\\c{g}": "\u0123",
    "\\c{K}": "\u0136",
    "\\c{k}": "\u0137",
    "\\c{L}": "\u013b",
    "\\c{l}": "\u013c",
    "\\c{N}": "\u0145",
    "\\c{n}": "\u0146",
    "\\c{R}": "\u0156",
    "\\c{r}": "\u0157",
    "\\c{S}": "\u015e",
    "\\c{s}": "\u015f",
    "\\c{T}": "\u0162",
    "\\c{t}": "\u0163",
    "\\cancer ": "\u264b",
    "\\cap ": "\u2229",
    "\\Cap ": "\u22d2",
    "\\capricornus ": "\u2651",
    "\\cdot ": "\u00b7",
    "\\cdots ": "\u22ef",
    "\\centerdot ": "\u22c5",
    "\\cents ": "\u00a2",
    "\\Chi ": "\u03a7",
    "\\chi ": "\u03c7",
    "\\chi_t": "\u03c7t",
    "\\circ ": "\u2218",
    "\\circeq ": "\u2257",
    "\\circlearrowleft ": "\u21ba",
    "\\circlearrowright ": "\u21bb",
    "\\circledast ": "\u229b",
    "\\circledcirc ": "\u229a",
    "\\circleddash ": "\u229d",
    "\\circledS ": "\u24c8",
    "\\clockoint ": "\u2a0f",
    "\\clubsuit ": "\u2663",
    "\\clwintegral ": "\u2231",
    "\\Colon ": "\u2237",
    "\\complement ": "\u2201",
    "\\coprod ": "\u2210",
    "\\copyright ": "\u00a9",
    "\\cup ": "\u222a",
    "\\Cup ": "\u22d3",
    "\\curlyeqprec ": "\u22de",
    "\\curlyeqsucc ": "\u22df",
    "\\curlyvee ": "\u22ce",
    "\\curlywedge ": "\u22cf",
    "\\curvearrowleft ": "\u21b6",
    "\\curvearrowright ": "\u21b7",
    "\\cyrchar{\\'\\CYRG}": "\u0403",
    "\\cyrchar{\\'\\cyrg}": "\u0453",
    "\\cyrchar{\\'\\CYRK}": "\u040c",
    "\\cyrchar{\\'\\cyrk}": "\u045c",
    "\\cyrchar\\CYRA ": "\u0410",
    "\\cyrchar\\cyra ": "\u0430",
    "\\cyrchar\\CYRABHCH ": "\u04bc",
    "\\cyrchar\\cyrabhch ": "\u04bd",
    "\\cyrchar\\CYRABHCHDSC ": "\u04be",
    "\\cyrchar\\cyrabhchdsc ": "\u04bf",
    "\\cyrchar\\CYRABHDZE ": "\u04e0",
    "\\cyrchar\\cyrabhdze ": "\u04e1",
    "\\cyrchar\\CYRABHHA ": "\u04a8",
    "\\cyrchar\\cyrabhha ": "\u04a9",
    "\\cyrchar\\CYRAE ": "\u04d4",
    "\\cyrchar\\cyrae ": "\u04d5",
    "\\cyrchar\\CYRB ": "\u0411",
    "\\cyrchar\\cyrb ": "\u0431",
    "\\cyrchar\\CYRBYUS ": "\u046a",
    "\\cyrchar\\CYRC ": "\u0426",
    "\\cyrchar\\cyrc ": "\u0446",
    "\\cyrchar\\CYRCH ": "\u0427",
    "\\cyrchar\\cyrch ": "\u0447",
    "\\cyrchar\\CYRCHLDSC ": "\u04cb",
    "\\cyrchar\\cyrchldsc ": "\u04cc",
    "\\cyrchar\\CYRCHRDSC ": "\u04b6",
    "\\cyrchar\\cyrchrdsc ": "\u04b7",
    "\\cyrchar\\CYRCHVCRS ": "\u04b8",
    "\\cyrchar\\cyrchvcrs ": "\u04b9",
    "\\cyrchar\\CYRD ": "\u0414",
    "\\cyrchar\\cyrd ": "\u0434",
    "\\cyrchar\\CYRDJE ": "\u0402",
    "\\cyrchar\\cyrdje ": "\u0452",
    "\\cyrchar\\CYRDZE ": "\u0405",
    "\\cyrchar\\cyrdze ": "\u0455",
    "\\cyrchar\\CYRDZHE ": "\u040f",
    "\\cyrchar\\cyrdzhe ": "\u045f",
    "\\cyrchar\\CYRE ": "\u0415",
    "\\cyrchar\\cyre ": "\u0435",
    "\\cyrchar\\CYREREV ": "\u042d",
    "\\cyrchar\\cyrerev ": "\u044d",
    "\\cyrchar\\CYRERY ": "\u042b",
    "\\cyrchar\\cyrery ": "\u044b",
    "\\cyrchar\\CYRF ": "\u0424",
    "\\cyrchar\\cyrf ": "\u0444",
    "\\cyrchar\\CYRFITA ": "\u0472",
    "\\cyrchar\\CYRG ": "\u0413",
    "\\cyrchar\\cyrg ": "\u0433",
    "\\cyrchar\\CYRGHCRS ": "\u0492",
    "\\cyrchar\\cyrghcrs ": "\u0493",
    "\\cyrchar\\CYRGHK ": "\u0494",
    "\\cyrchar\\cyrghk ": "\u0495",
    "\\cyrchar\\CYRGUP ": "\u0490",
    "\\cyrchar\\cyrgup ": "\u0491",
    "\\cyrchar\\CYRH ": "\u0425",
    "\\cyrchar\\cyrh ": "\u0445",
    "\\cyrchar\\CYRHDSC ": "\u04b2",
    "\\cyrchar\\cyrhdsc ": "\u04b3",
    "\\cyrchar\\CYRHRDSN ": "\u042a",
    "\\cyrchar\\cyrhrdsn ": "\u044a",
    "\\cyrchar\\cyrhundredthousands ": "\u0488",
    "\\cyrchar\\CYRI ": "\u0418",
    "\\cyrchar\\cyri ": "\u0438",
    "\\cyrchar\\CYRIE ": "\u0404",
    "\\cyrchar\\cyrie ": "\u0454",
    "\\cyrchar\\CYRII ": "\u0406",
    "\\cyrchar\\cyrii ": "\u0456",
    "\\cyrchar\\CYRIOTBYUS ": "\u046c",
    "\\cyrchar\\cyriotbyus ": "\u046d",
    "\\cyrchar\\CYRIOTE ": "\u0464",
    "\\cyrchar\\cyriote ": "\u0465",
    "\\cyrchar\\CYRIOTLYUS ": "\u0468",
    "\\cyrchar\\cyriotlyus ": "\u0469",
    "\\cyrchar\\CYRISHRT ": "\u0419",
    "\\cyrchar\\cyrishrt ": "\u0439",
    "\\cyrchar\\CYRIZH ": "\u0474",
    "\\cyrchar\\CYRJE ": "\u0408",
    "\\cyrchar\\cyrje ": "\u0458",
    "\\cyrchar\\CYRK ": "\u041a",
    "\\cyrchar\\cyrk ": "\u043a",
    "\\cyrchar\\CYRKBEAK ": "\u04a0",
    "\\cyrchar\\cyrkbeak ": "\u04a1",
    "\\cyrchar\\CYRKDSC ": "\u049a",
    "\\cyrchar\\cyrkdsc ": "\u049b",
    "\\cyrchar\\CYRKHCRS ": "\u049e",
    "\\cyrchar\\cyrkhcrs ": "\u049f",
    "\\cyrchar\\CYRKHK ": "\u04c3",
    "\\cyrchar\\cyrkhk ": "\u04c4",
    "\\cyrchar\\CYRKOPPA ": "\u0480",
    "\\cyrchar\\cyrkoppa ": "\u0481",
    "\\cyrchar\\CYRKSI ": "\u046e",
    "\\cyrchar\\cyrksi ": "\u046f",
    "\\cyrchar\\CYRKVCRS ": "\u049c",
    "\\cyrchar\\cyrkvcrs ": "\u049d",
    "\\cyrchar\\CYRL ": "\u041b",
    "\\cyrchar\\cyrl ": "\u043b",
    "\\cyrchar\\CYRLJE ": "\u0409",
    "\\cyrchar\\cyrlje ": "\u0459",
    "\\cyrchar\\CYRLYUS ": "\u0466",
    "\\cyrchar\\cyrlyus ": "\u0467",
    "\\cyrchar\\CYRM ": "\u041c",
    "\\cyrchar\\cyrm ": "\u043c",
    "\\cyrchar\\cyrmillions ": "\u0489",
    "\\cyrchar\\CYRN ": "\u041d",
    "\\cyrchar\\cyrn ": "\u043d",
    "\\cyrchar\\CYRNDSC ": "\u04a2",
    "\\cyrchar\\cyrndsc ": "\u04a3",
    "\\cyrchar\\CYRNG ": "\u04a4",
    "\\cyrchar\\cyrng ": "\u04a5",
    "\\cyrchar\\CYRNHK ": "\u04c7",
    "\\cyrchar\\cyrnhk ": "\u04c8",
    "\\cyrchar\\CYRNJE ": "\u040a",
    "\\cyrchar\\cyrnje ": "\u045a",
    "\\cyrchar\\CYRO ": "\u041e",
    "\\cyrchar\\cyro ": "\u043e",
    "\\cyrchar\\CYROMEGA ": "\u0460",
    "\\cyrchar\\cyromega ": "\u0461",
    "\\cyrchar\\CYROMEGARND ": "\u047a",
    "\\cyrchar\\cyromegarnd ": "\u047b",
    "\\cyrchar\\CYROMEGATITLO ": "\u047c",
    "\\cyrchar\\cyromegatitlo ": "\u047d",
    "\\cyrchar\\CYROT ": "\u047e",
    "\\cyrchar\\cyrot ": "\u047f",
    "\\cyrchar\\CYROTLD ": "\u04e8",
    "\\cyrchar\\cyrotld ": "\u04e9",
    "\\cyrchar\\CYRP ": "\u041f",
    "\\cyrchar\\cyrp ": "\u043f",
    "\\cyrchar\\CYRpalochka ": "\u04c0",
    "\\cyrchar\\CYRPHK ": "\u04a6",
    "\\cyrchar\\cyrphk ": "\u04a7",
    "\\cyrchar\\CYRPSI ": "\u0470",
    "\\cyrchar\\cyrpsi ": "\u0471",
    "\\cyrchar\\CYRR ": "\u0420",
    "\\cyrchar\\cyrr ": "\u0440",
    "\\cyrchar\\CYRRTICK ": "\u048e",
    "\\cyrchar\\cyrrtick ": "\u048f",
    "\\cyrchar\\CYRS ": "\u0421",
    "\\cyrchar\\cyrs ": "\u0441",
    "\\cyrchar\\CYRSCHWA ": "\u04d8",
    "\\cyrchar\\cyrschwa ": "\u04d9",
    "\\cyrchar\\CYRSDSC ": "\u04aa",
    "\\cyrchar\\cyrsdsc ": "\u04ab",
    "\\cyrchar\\CYRSEMISFTSN ": "\u048c",
    "\\cyrchar\\cyrsemisftsn ": "\u048d",
    "\\cyrchar\\CYRSFTSN ": "\u042c",
    "\\cyrchar\\cyrsftsn ": "\u044c",
    "\\cyrchar\\CYRSH ": "\u0428",
    "\\cyrchar\\cyrsh ": "\u0448",
    "\\cyrchar\\CYRSHCH ": "\u0429",
    "\\cyrchar\\cyrshch ": "\u0449",
    "\\cyrchar\\CYRSHHA ": "\u04ba",
    "\\cyrchar\\cyrshha ": "\u04bb",
    "\\cyrchar\\CYRT ": "\u0422",
    "\\cyrchar\\cyrt ": "\u0442",
    "\\cyrchar\\CYRTDSC ": "\u04ac",
    "\\cyrchar\\cyrtdsc ": "\u04ad",
    "\\cyrchar\\CYRTETSE ": "\u04b4",
    "\\cyrchar\\cyrtetse ": "\u04b5",
    "\\cyrchar\\cyrthousands ": "\u0482",
    "\\cyrchar\\CYRTSHE ": "\u040b",
    "\\cyrchar\\cyrtshe ": "\u045b",
    "\\cyrchar\\CYRU ": "\u0423",
    "\\cyrchar\\cyru ": "\u0443",
    "\\cyrchar\\CYRUK ": "\u0478",
    "\\cyrchar\\cyruk ": "\u0479",
    "\\cyrchar\\CYRUSHRT ": "\u040e",
    "\\cyrchar\\cyrushrt ": "\u045e",
    "\\cyrchar\\CYRV ": "\u0412",
    "\\cyrchar\\cyrv ": "\u0432",
    "\\cyrchar\\CYRY ": "\u04ae",
    "\\cyrchar\\cyry ": "\u04af",
    "\\cyrchar\\CYRYA ": "\u042f",
    "\\cyrchar\\cyrya ": "\u044f",
    "\\cyrchar\\CYRYAT ": "\u0462",
    "\\cyrchar\\CYRYHCRS ": "\u04b0",
    "\\cyrchar\\cyryhcrs ": "\u04b1",
    "\\cyrchar\\CYRYI ": "\u0407",
    "\\cyrchar\\cyryi ": "\u0457",
    "\\cyrchar\\CYRYO ": "\u0401",
    "\\cyrchar\\cyryo ": "\u0451",
    "\\cyrchar\\CYRYU ": "\u042e",
    "\\cyrchar\\cyryu ": "\u044e",
    "\\cyrchar\\CYRZ ": "\u0417",
    "\\cyrchar\\cyrz ": "\u0437",
    "\\cyrchar\\CYRZDSC ": "\u0498",
    "\\cyrchar\\cyrzdsc ": "\u0499",
    "\\cyrchar\\CYRZH ": "\u0416",
    "\\cyrchar\\cyrzh ": "\u0436",
    "\\cyrchar\\CYRZHDSC ": "\u0496",
    "\\cyrchar\\cyrzhdsc ": "\u0497",
    "\\cyrchar\\textnumero ": "\u2116",
    "\\dagger ": "\u2020",
    "\\dashleftarrow ": "\u21e0",
    "\\dashrightarrow ": "\u21e2",
    "\\ddagger ": "\u2021",
    "\\daleth ": "\u2138",
    "\\dashv ": "\u22a3",
    "\\dblarrowupdown ": "\u21c5",
    "\\ddddot ": "\u20dc",
    "\\dddot ": "\u20db",
    "\\ddots ": "\u22f1",
    "\\degree ": "\u00b0",
    "\\Delta ": "\u0394",
    "\\delta ": "\u03b4",
    "\\DH ": "\u00d0",
    "\\dh ": "\u00f0",
    "\\diagup ": "\u2571",
    "\\diamond ": "\u22c4",
    "\\diamondsuit ": "\u2662",
    "\\Digamma ": "\u03dc",
    "\\digamma ": "\u03dd",
    "\\ding{100}": "\u2744",
    "\\ding{101}": "\u2745",
    "\\ding{102}": "\u2746",
    "\\ding{103}": "\u2747",
    "\\ding{104}": "\u2748",
    "\\ding{105}": "\u2749",
    "\\ding{106}": "\u274a",
    "\\ding{107}": "\u274b",
    "\\ding{108}": "\u25cf",
    "\\ding{109}": "\u274d",
    "\\ding{110}": "\u25a0",
    "\\ding{111}": "\u274f",
    "\\ding{112}": "\u2750",
    "\\ding{113}": "\u2751",
    "\\ding{114}": "\u2752",
    "\\ding{115}": "\u25b2",
    "\\ding{116}": "\u25bc",
    "\\ding{117}": "\u25c6",
    "\\ding{118}": "\u2756",
    "\\ding{119}": "\u25d7",
    "\\ding{120}": "\u2758",
    "\\ding{121}": "\u2759",
    "\\ding{122}": "\u275a",
    "\\ding{123}": "\u275b",
    "\\ding{124}": "\u275c",
    "\\ding{125}": "\u275d",
    "\\ding{126}": "\u275e",
    "\\ding{161}": "\u2761",
    "\\ding{162}": "\u2762",
    "\\ding{163}": "\u2763",
    "\\ding{164}": "\u2764",
    "\\ding{165}": "\u2765",
    "\\ding{166}": "\u2766",
    "\\ding{167}": "\u2767",
    "\\ding{168}": "\u2663",
    "\\ding{169}": "\u2666",
    "\\ding{170}": "\u2665",
    "\\ding{171}": "\u2660",
    "\\ding{172}": "\u2460",
    "\\ding{173}": "\u2461",
    "\\ding{174}": "\u2462",
    "\\ding{175}": "\u2463",
    "\\ding{176}": "\u2464",
    "\\ding{177}": "\u2465",
    "\\ding{178}": "\u2466",
    "\\ding{179}": "\u2467",
    "\\ding{180}": "\u2468",
    "\\ding{181}": "\u2469",
    "\\ding{182}": "\u2776",
    "\\ding{183}": "\u2777",
    "\\ding{184}": "\u2778",
    "\\ding{185}": "\u2779",
    "\\ding{186}": "\u277a",
    "\\ding{187}": "\u277b",
    "\\ding{188}": "\u277c",
    "\\ding{189}": "\u277d",
    "\\ding{190}": "\u277e",
    "\\ding{191}": "\u277f",
    "\\ding{192}": "\u2780",
    "\\ding{193}": "\u2781",
    "\\ding{194}": "\u2782",
    "\\ding{195}": "\u2783",
    "\\ding{196}": "\u2784",
    "\\ding{197}": "\u2785",
    "\\ding{198}": "\u2786",
    "\\ding{199}": "\u2787",
    "\\ding{200}": "\u2788",
    "\\ding{201}": "\u2789",
    "\\ding{202}": "\u278a",
    "\\ding{203}": "\u278b",
    "\\ding{204}": "\u278c",
    "\\ding{205}": "\u278d",
    "\\ding{206}": "\u278e",
    "\\ding{207}": "\u278f",
    "\\ding{208}": "\u2790",
    "\\ding{209}": "\u2791",
    "\\ding{210}": "\u2792",
    "\\ding{211}": "\u2793",
    "\\ding{212}": "\u2794",
    "\\ding{216}": "\u2798",
    "\\ding{217}": "\u2799",
    "\\ding{218}": "\u279a",
    "\\ding{219}": "\u279b",
    "\\ding{220}": "\u279c",
    "\\ding{221}": "\u279d",
    "\\ding{222}": "\u279e",
    "\\ding{223}": "\u279f",
    "\\ding{224}": "\u27a0",
    "\\ding{225}": "\u27a1",
    "\\ding{226}": "\u27a2",
    "\\ding{227}": "\u27a3",
    "\\ding{228}": "\u27a4",
    "\\ding{229}": "\u27a5",
    "\\ding{230}": "\u27a6",
    "\\ding{231}": "\u27a7",
    "\\ding{232}": "\u27a8",
    "\\ding{233}": "\u27a9",
    "\\ding{234}": "\u27aa",
    "\\ding{235}": "\u27ab",
    "\\ding{236}": "\u27ac",
    "\\ding{237}": "\u27ad",
    "\\ding{238}": "\u27ae",
    "\\ding{239}": "\u27af",
    "\\ding{241}": "\u27b1",
    "\\ding{242}": "\u27b2",
    "\\ding{243}": "\u27b3",
    "\\ding{244}": "\u27b4",
    "\\ding{245}": "\u27b5",
    "\\ding{246}": "\u27b6",
    "\\ding{247}": "\u27b7",
    "\\ding{248}": "\u27b8",
    "\\ding{249}": "\u27b9",
    "\\ding{250}": "\u27ba",
    "\\ding{251}": "\u27bb",
    "\\ding{252}": "\u27bc",
    "\\ding{253}": "\u27bd",
    "\\ding{254}": "\u27be",
    "\\ding{33}": "\u2701",
    "\\ding{34}": "\u2702",
    "\\ding{35}": "\u2703",
    "\\ding{36}": "\u2704",
    "\\ding{37}": "\u260e",
    "\\ding{38}": "\u2706",
    "\\ding{39}": "\u2707",
    "\\ding{40}": "\u2708",
    "\\ding{41}": "\u2709",
    "\\ding{42}": "\u261b",
    "\\ding{43}": "\u261e",
    "\\ding{44}": "\u270c",
    "\\ding{45}": "\u270d",
    "\\ding{46}": "\u270e",
    "\\ding{47}": "\u270f",
    "\\ding{48}": "\u2710",
    "\\ding{49}": "\u2711",
    "\\ding{50}": "\u2712",
    "\\ding{51}": "\u2713",
    "\\ding{52}": "\u2714",
    "\\ding{53}": "\u2715",
    "\\ding{54}": "\u2716",
    "\\ding{55}": "\u2717",
    "\\ding{56}": "\u2718",
    "\\ding{57}": "\u2719",
    "\\ding{58}": "\u271a",
    "\\ding{59}": "\u271b",
    "\\ding{60}": "\u271c",
    "\\ding{61}": "\u271d",
    "\\ding{62}": "\u271e",
    "\\ding{63}": "\u271f",
    "\\ding{64}": "\u2720",
    "\\ding{65}": "\u2721",
    "\\ding{66}": "\u2722",
    "\\ding{67}": "\u2723",
    "\\ding{68}": "\u2724",
    "\\ding{69}": "\u2725",
    "\\ding{70}": "\u2726",
    "\\ding{71}": "\u2727",
    "\\ding{72}": "\u2605",
    "\\ding{73}": "\u2606",
    "\\ding{74}": "\u272a",
    "\\ding{75}": "\u272b",
    "\\ding{76}": "\u272c",
    "\\ding{77}": "\u272d",
    "\\ding{78}": "\u272e",
    "\\ding{79}": "\u272f",
    "\\ding{80}": "\u2730",
    "\\ding{81}": "\u2731",
    "\\ding{82}": "\u2732",
    "\\ding{83}": "\u2733",
    "\\ding{84}": "\u2734",
    "\\ding{85}": "\u2735",
    "\\ding{86}": "\u2736",
    "\\ding{87}": "\u2737",
    "\\ding{88}": "\u2738",
    "\\ding{89}": "\u2739",
    "\\ding{90}": "\u273a",
    "\\ding{91}": "\u273b",
    "\\ding{92}": "\u273c",
    "\\ding{93}": "\u273d",
    "\\ding{94}": "\u273e",
    "\\ding{95}": "\u273f",
    "\\ding{96}": "\u2740",
    "\\ding{97}": "\u2741",
    "\\ding{98}": "\u2742",
    "\\ding{99}": "\u2743",
    "\\div ": "\u00f7",
    "\\divideontimes ": "\u22c7",
    "\\DJ ": "\u0110",
    "\\dj ": "\u0111",
    "\\doteq ": "\u2250",
    "\\doteqdot ": "\u2251",
    "\\dotplus ": "\u2214",
    "\\dots ": "\u2026",
    "\\downarrow ": "\u2193",
    "\\Downarrow ": "\u21d3",
    "\\DownArrowBar ": "\u2913",
    "\\DownArrowUpArrow ": "\u21f5",
    "\\downdownarrows ": "\u21ca",
    "\\downharpoonleft ": "\u21c3",
    "\\downharpoonright ": "\u21c2",
    "\\DownLeftRightVector ": "\u2950",
    "\\DownLeftTeeVector ": "\u295e",
    "\\DownLeftVectorBar ": "\u2956",
    "\\DownRightTeeVector ": "\u295f",
    "\\DownRightVectorBar ": "\u2957",
    "\\downslopeellipsis ": "\u22f1",
    "\\eighthnote ": "\u266a",
    "\\ell ": "\u2113",
    "\\Elolarr ": "\u2940",
    "\\ElOr ": "\u2a56",
    "\\Elorarr ": "\u2941",
    "\\Elroang ": "\u2986",
    "\\ElsevierGlyph{2129}": "\u2129",
    "\\ElsevierGlyph{21B3}": "\u21b3",
    "\\ElsevierGlyph{2232}": "\u2232",
    "\\ElsevierGlyph{2233}": "\u2233",
    "\\ElsevierGlyph{2238}": "\u2238",
    "\\ElsevierGlyph{2242}": "\u2242",
    "\\ElsevierGlyph{225A}": "\u225a",
    "\\ElsevierGlyph{225F}": "\u225f",
    "\\ElsevierGlyph{2274}": "\u2274",
    "\\ElsevierGlyph{2275}": "\u2275",
    "\\ElsevierGlyph{22C0}": "\u22c0",
    "\\ElsevierGlyph{22C1}": "\u22c1",
    "\\ElsevierGlyph{300A}": "\u300a",
    "\\ElsevierGlyph{300B}": "\u300b",
    "\\ElsevierGlyph{3018}": "\u3018",
    "\\ElsevierGlyph{3019}": "\u3019",
    "\\ElsevierGlyph{E20A}": "\u2926",
    "\\ElsevierGlyph{E20B}": "\u2925",
    "\\ElsevierGlyph{E20C}": "\u2923",
    "\\ElsevierGlyph{E20D}": "\u2924",
    "\\ElsevierGlyph{E20E}": "\u2928",
    "\\ElsevierGlyph{E20F}": "\u2929",
    "\\ElsevierGlyph{E210}": "\u292a",
    "\\ElsevierGlyph{E211}": "\u2927",
    "\\ElsevierGlyph{E212}": "\u2905",
    "\\ElsevierGlyph{E214}": "\u297c",
    "\\ElsevierGlyph{E215}": "\u297d",
    "\\ElsevierGlyph{E219}": "\u2937",
    "\\ElsevierGlyph{E21A}": "\u2936",
    "\\ElsevierGlyph{E21C}": "\u2933",
    "\\ElsevierGlyph{E21D}": "\u2933\u0338",
    "\\ElsevierGlyph{E259}": "\u2a3c",
    "\\ElsevierGlyph{E25A}": "\u2a25",
    "\\ElsevierGlyph{E25B}": "\u2a2a",
    "\\ElsevierGlyph{E25C}": "\u2a2d",
    "\\ElsevierGlyph{E25D}": "\u2a2e",
    "\\ElsevierGlyph{E25E}": "\u2a35",
    "\\ElsevierGlyph{E260}": "\u29b5",
    "\\ElsevierGlyph{E291}": "\u2994",
    "\\ElsevierGlyph{E30D}": "\u2aeb",
    "\\ElsevierGlyph{E36E}": "\u2a55",
    "\\ElsevierGlyph{E372}": "\u29dc",
    "\\ElsevierGlyph{E381}": "\u25b1",
    "\\ElsevierGlyph{E395}": "\u2a10",
    "\\ElsevierGlyph{E61B}": "\u29b6",
    "\\ElsevierGlyph{E838}": "\u233d",
    "\\Elxsqcup ": "\u2a06",
    "\\Elxuplus ": "\u2a04",
    "\\ElzAnd ": "\u2a53",
    "\\Elzbar ": "\u0336",
    "\\Elzbtdl ": "\u026c",
    "\\ElzCint ": "\u2a0d",
    "\\Elzcirfb ": "\u25d2",
    "\\Elzcirfl ": "\u25d0",
    "\\Elzcirfr ": "\u25d1",
    "\\Elzclomeg ": "\u0277",
    "\\Elzddfnc ": "\u2999",
    "\\Elzdefas ": "\u29cb",
    "\\Elzdlcorn ": "\u23a3",
    "\\Elzdshfnc ": "\u2506",
    "\\Elzdyogh ": "\u02a4",
    "\\Elzesh ": "\u0283",
    "\\Elzfhr ": "\u027e",
    "\\Elzglst ": "\u0294",
    "\\Elzhlmrk ": "\u02d1",
    "\\ElzInf ": "\u2a07",
    "\\Elzinglst ": "\u0296",
    "\\Elzinvv ": "\u028c",
    "\\Elzinvw ": "\u028d",
    "\\ElzLap ": "\u29ca",
    "\\Elzlmrk ": "\u02d0",
    "\\Elzlow ": "\u02d5",
    "\\Elzlpargt ": "\u29a0",
    "\\Elzltlmr ": "\u0271",
    "\\Elzltln ": "\u0272",
    "\\Elzminhat ": "\u2a5f",
    "\\Elzopeno ": "\u0254",
    "\\ElzOr ": "\u2a54",
    "\\Elzpalh ": "\u0321",
    "\\Elzpbgam ": "\u0264",
    "\\Elzpgamma ": "\u0263",
    "\\Elzpscrv ": "\u028b",
    "\\Elzpupsil ": "\u028a",
    "\\Elzrais ": "\u02d4",
    "\\Elzrarrx ": "\u2947",
    "\\Elzreapos ": "\u201b",
    "\\Elzreglst ": "\u0295",
    "\\Elzrh ": "\u0322",
    "\\Elzrl ": "\u027c",
    "\\ElzRlarr ": "\u2942",
    "\\ElzrLarr ": "\u2944",
    "\\Elzrtld ": "\u0256",
    "\\Elzrtll ": "\u026d",
    "\\Elzrtln ": "\u0273",
    "\\Elzrtlr ": "\u027d",
    "\\Elzrtls ": "\u0282",
    "\\Elzrtlt ": "\u0288",
    "\\Elzrtlz ": "\u0290",
    "\\Elzrttrnr ": "\u027b",
    "\\Elzrvbull ": "\u25d8",
    "\\Elzsbbrg ": "\u032a",
    "\\Elzsblhr ": "\u02d3",
    "\\Elzsbrhr ": "\u02d2",
    "\\Elzschwa ": "\u0259",
    "\\Elzsqfl ": "\u25e7",
    "\\Elzsqfnw ": "\u2519",
    "\\Elzsqfr ": "\u25e8",
    "\\Elzsqfse ": "\u25ea",
    "\\Elzsqspne ": "\u22e5",
    "\\ElzSup ": "\u2a08",
    "\\Elztdcol ": "\u2af6",
    "\\Elztesh ": "\u02a7",
    "\\Elztfnc ": "\u2980",
    "\\ElzThr ": "\u2a05",
    "\\ElzTimes ": "\u2a2f",
    "\\Elztrna ": "\u0250",
    "\\Elztrnh ": "\u0265",
    "\\Elztrnm ": "\u026f",
    "\\Elztrnmlr ": "\u0270",
    "\\Elztrnr ": "\u0279",
    "\\Elztrnrl ": "\u027a",
    "\\Elztrnsa ": "\u0252",
    "\\Elztrnt ": "\u0287",
    "\\Elztrny ": "\u028e",
    "\\Elzverti ": "\u02cc",
    "\\Elzverts ": "\u02c8",
    "\\Elzvrecto ": "\u25af",
    "\\Elzxh ": "\u0127",
    "\\Elzxl ": "\u0335",
    "\\Elzxrat ": "\u211e",
    "\\Elzyogh ": "\u0292",
    "\\emdash ": "\u2014 ",
    "\\emptyset ": "\u2205",
    "\\ensuremath{\\Elzpes}": "\u20a7",
    "\\Epsilon ": "\u0395",
    "\\epsilon ": "\u03b5",
    "\\eqcirc ": "\u2256",
    "\\eqslantgtr ": "\u2a96",
    "\\eqslantless ": "\u2a95",
    "\\Equal ": "\u2a75",
    "\\estimates ": "\u2259",
    "\\Eta ": "\u0397",
    "\\eta ": "\u03b7",
    "\\eth ": "\u01aa",
    "\\euro ": "\u20ac",
    "\\exists ": "\u2203",
    "\\fallingdotseq ": "\u2252",
    "\\fbox{~~}": "\u25ad",
    "\\finv ": "\u2132",
    "\\flat ": "\u266d",
    "\\forall ": "\u2200",
    "\\forcesextra ": "\u22a8",
    "\\frown ": "\u2322",
    "\\Gamma ": "\u0393",
    "\\gamma ": "\u03b3",
    "\\gemini ": "\u264a",
    "\\geqq ": "\u2267",
    "\\geqslant ": "\u2a7e",
    "\\gg ": "\u226b",
    "\\ggg ": "\u22d9",
    "\\gimel ": "\u2137",
    "\\gnapprox ": "\u2a8a",
    "\\gneq ": "\u2a88",
    "\\gneqq ": "\u2269",
    "\\gnsim ": "\u22e7",
    "\\greaterequivlnt ": "\u2273",
    "\\gtrapprox ": "\u2a86",
    "\\gtrdot ": "\u22d7",
    "\\gtreqless ": "\u22db",
    "\\gtreqqless ": "\u2a8c",
    "\\gtrless ": "\u2277",
    "\\gtrsim ": "\u2273",
    "\\guillemotleft ": "\u00ab",
    "\\guillemotright ": "\u00bb",
    "\\guilsinglleft ": "\u2039",
    "\\guilsinglright ": "\u203a",
    "\\gvertneqq ": "\u2269\ufe00",
    "\\H{}": "\u02dd",
    "\\H{O}": "\u0150",
    "\\H{o}": "\u0151",
    "\\H{U}": "\u0170",
    "\\H{u}": "\u0171",
    "\\hbar ": "\u210f",
    "\\heartsuit ": "\u2661",
    "\\hermitconjmatrix ": "\u22b9",
    "\\homothetic ": "\u223b",
    "\\hookleftarrow ": "\u21a9",
    "\\hookrightarrow ": "\u21aa",
    "\\hphantom{,}": "\u2008",
    "\\hphantom{0}": "\u2007",
    "\\hslash ": "\u210f",
    "\\hspace{0.166em}": "\u2006",
    "\\hspace{0.167em}": "\u2009",
    "\\hspace{0.25em}": "\u2005",
    "\\hspace{0.33em}": "\u2004",
    "\\hspace{0.6em}": "\u2002",
    "\\hspace{1em}": "\u2003",
    "\\im ": "\u2111",
    "\\image ": "\u22b7",
    "\\implies ": "\u21d2",
    "\\infty ": "\u221e",
    "\\int\\!\\int\\!\\int ": "\u222d",
    "\\intercal ": "\u22ba",
    "\\Iota ": "\u0399",
    "\\iota ": "\u03b9",
    "\\join ": "\u22c8",
    "\\jupiter ": "\u2643",
    "\\k{}": "\u02db",
    "\\k{A}": "\u0104",
    "\\k{a}": "\u0105",
    "\\k{E}": "\u0118",
    "\\k{e}": "\u0119",
    "\\k{I}": "\u012e",
    "\\k{i}": "\u012f",
    "\\k{U}": "\u0172",
    "\\k{u}": "\u0173",
    "\\Kappa ": "\u039a",
    "\\kappa ": "\u03ba",
    "\\Koppa ": "\u03de",
    "\\L ": "\u0141",
    "\\l ": "\u0142",
    "\\Lambda ": "\u039b",
    "\\lambda ": "\u03bb",
    "\\langle ": "\u2329",
    "\\lazysinv ": "\u223e",
    "\\lbrace ": "\u007b",
    "\\lceil ": "\u2308",
    "\\ldots ": "\u2026",
    "\\le ": "\u2264",
    "\\leadsto ": "\u219d",
    "\\leftarrow ": "\u2190",
    "\\Leftarrow ": "\u21d0",
    "\\leftarrowtail ": "\u21a2",
    "\\LeftDownTeeVector ": "\u2961",
    "\\LeftDownVectorBar ": "\u2959",
    "\\leftharpoondown ": "\u21bd",
    "\\leftharpoonup ": "\u21bc",
    "\\leftleftarrows ": "\u21c7",
    "\\leftrightarrow ": "\u2194",
    "\\Leftrightarrow ": "\u21d4",
    "\\leftrightarrows ": "\u21c6",
    "\\leftrightharpoons ": "\u21cb",
    "\\leftrightsquigarrow ": "\u21ad",
    "\\LeftRightVector ": "\u294e",
    "\\LeftTeeVector ": "\u295a",
    "\\leftthreetimes ": "\u22cb",
    "\\LeftTriangleBar ": "\u29cf",
    "\\LeftUpDownVector ": "\u2951",
    "\\LeftUpTeeVector ": "\u2960",
    "\\LeftUpVectorBar ": "\u2958",
    "\\LeftVectorBar ": "\u2952",
    "\\leo ": "\u264c",
    "\\leqq ": "\u2266",
    "\\leqslant ": "\u2a7d",
    "\\lessapprox ": "\u2a85",
    "\\lessdot ": "\u22d6",
    "\\lesseqgtr ": "\u22da",
    "\\lesseqqgtr ": "\u2a8b",
    "\\lessequivlnt ": "\u2272",
    "\\lessgtr ": "\u2276",
    "\\lfloor ": "\u230a",
    "\\libra ": "\u264e",
    "\\ll ": "\u226a",
    "\\llcorner ": "\u231e",
    "\\Lleftarrow ": "\u21da",
    "\\lmoustache ": "\u23b0",
    "\\lnapprox ": "\u2a89",
    "\\lneq ": "\u2a87",
    "\\lneqq ": "\u2268",
    "\\lnot ": "\u00ac",
    "\\lnsim ": "\u22e6",
    "\\longleftarrow ": "\u27f5",
    "\\Longleftarrow ": "\u27f8",
    "\\longleftrightarrow ": "\u27f7",
    "\\Longleftrightarrow ": "\u27fa",
    "\\longmapsto ": "\u27fc",
    "\\longrightarrow ": "\u27f6",
    "\\Longrightarrow ": "\u27f9",
    "\\looparrowleft ": "\u21ab",
    "\\looparrowright ": "\u21ac",
    "\\lozenge ": "\u25ca",
    "\\lrcorner ": "\u231f",
    "\\Lsh ": "\u21b0",
    "\\ltimes ": "\u22c9",
    "\\lvertneqq ": "\u2268\ufe00",
    "\\male ": "\u2642",
    "\\mapsto ": "\u21a6",
    "\\mathbb{C}": "\u2102",
    "\\mathbb{H}": "\u210d",
    "\\mathbb{N}": "\u2115",
    "\\mathbb{P}": "\u2119",
    "\\mathbb{Q}": "\u211a",
    "\\mathbb{R}": "\u211d",
    "\\mathbb{Z}": "\u2124",
    "\\mathbin{{:}\\!\\!{-}\\!\\!{:}}": "\u223a",
    '\\mathchar"2208': "\u2316",
    "\\mathfrak{R}": "\u211c",
    "\\mathrm{'\\Omega}": "\u038f",
    "\\mathrm{'Y}": "\u038e",
    "\\mathrm{\\ddot{I}}": "\u03aa",
    "\\mathrm{\\ddot{Y}}": "\u03ab",
    "\\mathrm{\\mu}": "\u00b5",
    "\\mathscr{B}": "\u212c",
    "\\mathscr{e}": "\u212f",
    "\\mathscr{E}": "\u2130",
    "\\mathscr{F}": "\u2131",
    "\\mathscr{g}": "\u210a",
    "\\mathscr{H}": "\u210b",
    "\\mathscr{I}": "\u2110",
    "\\mathscr{L}": "\u2112",
    "\\mathscr{l}": "\u2113",
    "\\mathscr{M}": "\u2133",
    "\\mathscr{o}": "\u2134",
    "\\mathscr{R}": "\u211b",
    "\\mbox{\\texteuro} ": "\u20ac",
    "\\measuredangle ": "\u2221",
    "\\mercury ": "\u263f",
    "\\mho ": "\u2127",
    "\\mid ": "\u2223",
    "\\mkern1mu ": "\u200a",
    "\\mkern4mu ": "\u205f",
    "\\models ": "\u22a8",
    "\\mp ": "\u2213",
    "\\multimap ": "\u22b8",
    "\\nabla ": "\u2207",
    "\\natural ": "\u266e",
    "\\nearrow ": "\u2197",
    "\\neg ": "\u00ac",
    "\\neptune ": "\u2646",
    "\\neq ": "\u2260",
    "\\NestedGreaterGreater ": "\u2aa2",
    "\\NestedLessLess ": "\u2aa1",
    "\\nexists ": "\u2204",
    "\\NG ": "\u014a",
    "\\ng ": "\u014b",
    "\\ngeqslant ": "\u2a7e\u0338",
    "\\nleftarrow ": "\u219a",
    "\\nLeftarrow ": "\u21cd",
    "\\nleftrightarrow ": "\u21ae",
    "\\nLeftrightarrow ": "\u21ce",
    "\\nleqslant ": "\u2a7d\u0338",
    "\\nmid ": "\u2224",
    "\\nolinebreak ": "\u2060",
    "\\not =": "\u2260",
    "\\not\\apid ": "\u224b\u0338",
    "\\not\\approx ": "\u2249",
    "\\not\\cong ": "\u2247",
    "\\not\\doteq": "\u2250\u0338",
    "\\not\\equiv ": "\u2262",
    "\\not\\geq ": "\u2271",
    "\\not\\in ": "\u2209",
    "\\not\\kern-0.3em\\times ": "\u226d",
    "\\not\\leq ": "\u2270",
    "\\not\\ni ": "\u220c",
    "\\not\\prec ": "\u2280",
    "\\not\\preceq ": "\u2aaf\u0338",
    "\\not\\sim ": "\u2241",
    "\\not\\simeq ": "\u2244",
    "\\not\\sqsubseteq ": "\u22e2",
    "\\not\\sqsupseteq ": "\u22e3",
    "\\not\\subset ": "\u2284",
    "\\not\\subseteq ": "\u2288",
    "\\not\\succ ": "\u2281",
    "\\not\\succeq ": "\u2ab0\u0338",
    "\\not\\supset ": "\u2285",
    "\\not\\supseteq ": "\u2289",
    "\\not&gt;": "\u226f",
    "\\not&lt;": "\u226e",
    "\\NotEqualTilde ": "\u2242\u0338",
    "\\NotGreaterGreater ": "\u226b\u0338",
    "\\notgreaterless ": "\u2279",
    "\\NotHumpDownHump ": "\u224e\u0338",
    "\\NotHumpEqual ": "\u224f\u0338",
    "\\notin ": "\u2209",
    "\\NotLeftTriangleBar ": "\u29cf\u0338",
    "\\notlessgreater ": "\u2278",
    "\\NotLessLess ": "\u226a\u0338",
    "\\NotNestedGreaterGreater ": "\u2aa2\u0338",
    "\\NotNestedLessLess ": "\u2aa1\u0338",
    "\\NotPrecedesTilde ": "\u227e\u0338",
    "\\NotRightTriangleBar ": "\u29d0\u0338",
    "\\NotSquareSubset ": "\u228f\u0338",
    "\\NotSquareSuperset ": "\u2290\u0338",
    "\\NotSucceedsTilde ": "\u227f\u0338",
    "\\nparallel ": "\u2226",
    "\\nrightarrow ": "\u219b",
    "\\nRightarrow ": "\u21cf",
    "\\nsubseteqq ": "\u2ac5\u0338",
    "\\nsupseteqq": "\u2ac6\u0338",
    "\\ntriangleleft ": "\u22ea",
    "\\ntrianglelefteq ": "\u22ec",
    "\\ntriangleright ": "\u22eb",
    "\\ntrianglerighteq ": "\u22ed",
    "\\nu ": "\u03bd",
    "\\nvdash ": "\u22ac",
    "\\nvDash ": "\u22ad",
    "\\nVdash ": "\u22ae",
    "\\nVDash ": "\u22af",
    "\\nwarrow ": "\u2196",
    "\\odot ": "\u2299",
    "\\OE ": "\u0152",
    "\\oe ": "\u0153",
    "\\oint ": "\u222e",
    "\\Omega ": "\u03a9",
    "\\omega ": "\u03c9",
    "\\ominus ": "\u2296",
    "\\openbracketleft ": "\u301a",
    "\\openbracketright ": "\u301b",
    "\\oplus ": "\u2295",
    "\\original ": "\u22b6",
    "\\oslash ": "\u2298",
    "\\otimes ": "\u2297",
    "\\parallel ": "\u2225",
    "\\partial ": "\u2202",
    "\\perp ": "\u22a5",
    "\\perspcorrespond ": "\u2a5e",
    "\\Phi ": "\u03a6",
    "\\phi ": "\u03d5",
    "\\Pi ": "\u03a0",
    "\\pi ": "\u03c0",
    "\\pilcrow ": "\u00b6",
    "\\pisces ": "\u2653",
    "\\Pisymbol{ppi020}{105}": "\u2a9e",
    "\\Pisymbol{ppi020}{117}": "\u2a9d",
    "\\Pisymbol{ppi022}{87}": "\u03d0",
    "\\pitchfork ": "\u22d4",
    "\\pluto ": "\u2647",
    "\\pm ": "\u00b1",
    "\\pound ": "\u00a3",
    "\\precapprox ": "\u2ab7",
    "\\preccurlyeq ": "\u227c",
    "\\precedesnotsimilar ": "\u22e8",
    "\\precnapprox ": "\u2ab9",
    "\\precneqq ": "\u2ab5",
    "\\precsim ": "\u227e",
    "\\prime ": "\u2032",
    "\\prod ": "\u220f",
    "\\propto ": "\u221d",
    "\\Psi ": "\u03a8",
    "\\psi ": "\u03c8",
    "\\quarternote ": "\u2669",
    "\\r{}": "\u02da",
    "\\r{U}": "\u016e",
    "\\r{u}": "\u016f",
    "\\raisebox{.5ex}{\\scritpsize TM}": "\u2122",
    "\\rangle ": "\u232a",
    "\\rbrace ": "\u007d",
    "\\rceil ": "\u2309",
    "\\re ": "\u211c",
    "\\recorder ": "\u2315",
    "\\registered ": "\u00ae",
    "\\ReverseUpEquilibrium ": "\u296f",
    "\\rfloor ": "\u230b",
    "\\rhd ": "\u22b3",
    "\\Rho ": "\u03a1",
    "\\rho ": "\u03c1",
    "\\rightangle ": "\u221f",
    "\\rightanglearc ": "\u22be",
    "\\rightarrow ": "\u2192",
    "\\Rightarrow ": "\u21d2",
    "\\rightarrowtail ": "\u21a3",
    "\\RightDownTeeVector ": "\u295d",
    "\\RightDownVectorBar ": "\u2955",
    "\\rightharpoondown ": "\u21c1",
    "\\rightharpoonup ": "\u21c0",
    "\\rightleftarrows ": "\u21c4",
    "\\rightleftharpoons ": "\u21cc",
    "\\rightmoon ": "\u263e",
    "\\rightrightarrows ": "\u21c9",
    "\\rightsquigarrow ": "\u21dd",
    "\\RightTeeVector ": "\u295b",
    "\\rightthreetimes ": "\u22cc",
    "\\RightTriangleBar ": "\u29d0",
    "\\RightUpDownVector ": "\u294f",
    "\\RightUpTeeVector ": "\u295c",
    "\\RightUpVectorBar ": "\u2954",
    "\\RightVectorBar ": "\u2953",
    "\\risingdotseq ": "\u2253",
    "\\rmoustache ": "\u23b1",
    "\\RoundImplies ": "\u2970",
    "\\Rrightarrow ": "\u21db",
    "\\Rsh ": "\u21b1",
    "\\rtimes ": "\u22ca",
    "\\rule{1em}{1pt}": "\u2015",
    "\\RuleDelayed ": "\u29f4",
    "\\sagittarius ": "\u2650",
    "\\Sampi ": "\u03e0",
    "\\saturn ": "\u2644",
    "\\scorpio ": "\u264f",
    "\\searrow ": "\u2198",
    "\\section ": "\u00a7",
    "\\setminus ": "\u2216",
    "\\sharp ": "\u266f",
    "\\shortparallel ": "\u2225",
    "\\Sigma ": "\u03a3",
    "\\sigma ": "\u03c3",
    "\\sim\\joinrel\\leadsto": "\u27ff",
    "\\slash ": "\u002f",
    "\\smallfrown ": "\u2322",
    "\\smallsetminus ": "\u2216",
    "\\smallsmile ": "\u2323",
    "\\smile ": "\u2323",
    "\\space ": "\u0020",
    "\\spadesuit ": "\u2660",
    "\\sphericalangle ": "\u2222",
    "\\sqcap ": "\u2293",
    "\\sqcup ": "\u2294",
    "\\sqrint ": "\u2a16",
    "\\sqsubset ": "\u228f",
    "\\sqsupset ": "\u2290",
    "\\square ": "\u25a1",
    "\\ss ": "\u00df",
    "\\stackrel{*}{=}": "\u2a6e",
    "\\star ": "\u22c6",
    "\\starequal ": "\u225b",
    "\\Stigma ": "\u03da",
    "\\Subset ": "\u22d0",
    "\\subseteqq ": "\u2ac5",
    "\\subsetneq ": "\u228a",
    "\\subsetneqq ": "\u2acb",
    "\\succapprox ": "\u2ab8",
    "\\succcurlyeq ": "\u227d",
    "\\succnapprox ": "\u2aba",
    "\\succneqq ": "\u2ab6",
    "\\succnsim ": "\u22e9",
    "\\sum ": "\u2211",
    "\\Supset ": "\u22d1",
    "\\supseteqq ": "\u2ac6",
    "\\supsetneq ": "\u228b",
    "\\supsetneqq ": "\u2acc",
    "\\surd ": "\u221a",
    "\\surfintegral ": "\u222f",
    "\\swarrow ": "\u2199",
    "\\Tau ": "\u03a4",
    "\\tau ": "\u03c4",
    "\\taurus ": "\u2649",
    "\\textasciiacute ": "\u00b4",
    "\\textasciibreve ": "\u02d8",
    "\\textasciicaron ": "\u02c7",
    "\\textasciidieresis ": "\u00a8",
    "\\textasciigrave ": "\u0060",
    "\\textasciimacron ": "\u00af",
    "\\textasciitilde ": "\u007e",
    "\\textbackslash ": "\u005c",
    "\\textbrokenbar ": "\u00a6",
    "\\textbullet ": "\u2022",
    "\\textcent ": "\u00a2",
    "\\textcopyright ": "\u00a9",
    "\\textcurrency ": "\u00a4",
    "\\textdagger ": "\u2020",
    "\\textdaggerdbl ": "\u2021",
    "\\textdegree ": "\u00b0",
    "\\textdollar ": "\u0024",
    "\\textdoublepipe ": "\u01c2",
    "\\textemdash ": "\u2014 ",
    "\\textendash ": "\u2013 ",
    "\\textexclamdown ": "\u00a1",
    "\\textfrac{1}{3}": "\u2153",
    "\\textfrac{1}{5}": "\u2155",
    "\\textfrac{1}{6}": "\u2159",
    "\\textfrac{1}{8}": "\u215b",
    "\\textfrac{2}{3}": "\u2154",
    "\\textfrac{2}{5}": "\u2156",
    "\\textfrac{3}{5}": "\u2157",
    "\\textfrac{3}{8}": "\u215c",
    "\\textfrac{4}{5}": "\u2158",
    "\\textfrac{5}{6}": "\u215a",
    "\\textfrac{5}{8}": "\u215d",
    "\\textfrac{7}{8}": "\u215e",
    "\\texthvlig ": "\u0195",
    "\\textnrleg ": "\u019e",
    "\\textonehalf ": "\u00bd",
    "\\textonequarter ": "\u00bc",
    "\\textordfeminine ": "\u00aa",
    "\\textordmasculine ": "\u00ba",
    "\\textparagraph ": "\u00b6",
    "\\textperiodcentered ": "\u02d9",
    "\\textpertenthousand ": "\u2031",
    "\\textperthousand ": "\u2030",
    "\\textphi ": "\u0278",
    "\\textquestiondown ": "\u00bf",
    "\\textquotedblleft ": "\u201c",
    "\\textquotedblright ": "\u201d",
    "\\textquotesingle ": "\u0027",
    "\\textregistered ": "\u00ae",
    "\\textsection ": "\u00a7",
    "\\textsterling ": "\u00a3",
    "\\texttheta ": "\u03b8",
    "\\textTheta ": "\u03f4",
    "\\textthreequarters ": "\u00be",
    "\\texttildelow ": "\u02dc",
    "\\texttimes ": "\u00d7",
    "\\texttrademark ": "\u2122",
    "\\textturnk ": "\u029e",
    "\\textvartheta ": "\u03d1",
    "\\textvisiblespace ": "\u2423",
    "\\textyen ": "\u00a5",
    "\\TH ": "\u00de",
    "\\th ": "\u00fe",
    "\\therefore ": "\u2234",
    "\\theta ": "\u03b8",
    "\\Theta ": "\u0398",
    "\\thickapprox ": "\u2248",
    "\\thicksim ": "\u223c",
    "\\tildetrpl ": "\u224b",
    "\\tone{11}": "\u02e9",
    "\\tone{22}": "\u02e8",
    "\\tone{33}": "\u02e7",
    "\\tone{44}": "\u02e6",
    "\\tone{55}": "\u02e5",
    "\\top ": "\u22a4",
    "\\trademark ": "\u2122",
    "\\triangle ": "\u25b3",
    "\\triangledown ": "\u25bf",
    "\\triangleleft ": "\u25c3",
    "\\trianglelefteq ": "\u22b4",
    "\\triangleq ": "\u225c",
    "\\triangleright ": "\u25b9",
    "\\trianglerighteq ": "\u22b5",
    "\\truestate ": "\u22a7",
    "\\twoheadleftarrow ": "\u219e",
    "\\twoheadrightarrow ": "\u21a0",
    "\\u{\\i}": "\u012d",
    "\\u{A}": "\u0102",
    "\\u{a}": "\u0103",
    "\\u{E}": "\u0114",
    "\\u{e}": "\u0115",
    "\\u{G}": "\u011e",
    "\\u{g}": "\u011f",
    "\\u{I}": "\u012c",
    "\\u{O}": "\u014e",
    "\\u{o}": "\u014f",
    "\\u{U}": "\u016c",
    "\\u{u}": "\u016d",
    "\\ulcorner ": "\u231c",
    "\\unlhd ": "\u22b4",
    "\\unrhd ": "\u22b5",
    "\\uparrow ": "\u2191",
    "\\Uparrow ": "\u21d1",
    "\\UpArrowBar ": "\u2912",
    "\\updownarrow ": "\u2195",
    "\\Updownarrow ": "\u21d5",
    "\\UpEquilibrium ": "\u296e",
    "\\upharpoonleft ": "\u21bf",
    "\\upharpoonright ": "\u21be",
    "\\uplus ": "\u228e",
    "\\Upsilon ": "\u03a5",
    "\\upsilon ": "\u03c5",
    "\\upslopeellipsis ": "\u22f0",
    "\\upuparrows ": "\u21c8",
    "\\uranus ": "\u2645",
    "\\urcorner ": "\u231d",
    "\\user{}": "\u007e",
    "\\v{C}": "\u010c",
    "\\v{c}": "\u010d",
    "\\v{D}": "\u010e",
    "\\v{d}": "\u010f",
    "\\v{E}": "\u011a",
    "\\v{e}": "\u011b",
    "\\v{L}": "\u013d",
    "\\v{l}": "\u013e",
    "\\v{N}": "\u0147",
    "\\v{n}": "\u0148",
    "\\v{R}": "\u0158",
    "\\v{r}": "\u0159",
    "\\v{S}": "\u0160",
    "\\v{s}": "\u0161",
    "\\v{T}": "\u0164",
    "\\v{t}": "\u0165",
    "\\v{Z}": "\u017d",
    "\\v{z}": "\u017e",
    "\\varepsilon ": "\u025b",
    "\\varkappa ": "\u03f0",
    "\\varnothing ": "\u2205",
    "\\varphi ": "\u03c6",
    "\\varpi ": "\u03d6",
    "\\varpropto ": "\u221d",
    "\\varrho ": "\u03f1",
    "\\varsigma ": "\u03c2",
    "\\varsubsetneqq ": "\u228a\ufe00",
    "\\varsupsetneq ": "\u228b\ufe00",
    "\\vartheta ": "\u03d1",
    "\\vartriangle ": "\u25b5",
    "\\vartriangleleft ": "\u22b2",
    "\\vartriangleright ": "\u22b3",
    "\\vdash ": "\u22a2",
    "\\Vdash ": "\u22a9",
    "\\VDash ": "\u22ab",
    "\\vdots ": "\u22ee",
    "\\vee ": "\u2228",
    "\\veebar ": "\u22bb",
    "\\venus ": "\u2640",
    "\\vert ": "\u007c",
    "\\Vert ": "\u2016",
    "\\verymuchgreater ": "\u22d9",
    "\\verymuchless ": "\u22d8",
    "\\virgo ": "\u264d",
    "\\volintegral ": "\u2230",
    "\\Vvdash ": "\u22aa",
    "\\wedge ": "\u2227",
    "\\wp ": "\u2118",
    "\\wr ": "\u2240",
    "\\Xi ": "\u039e",
    "\\xi ": "\u03be",
    "\\Zeta ": "\u0396",
    "\\zeta ": "\u03b6",
    "&lt;\\kern-0.58em(": "\u2993",
    "=:": "\u2255",
    "\\log": "log",
    "\\sqrt": "\u221a",
}
